@@ -25,6 +25,10 @@
 #include <zlib.h>
 #include <locale.h>
 #include <time.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <stdio.h>
+#include <fcntl.h>
 
 #include <glib/gstdio.h>
 
@@ -35,7 +39,6 @@
 #include <libtracker-common/tracker-utils.h>
 
 #include "tracker-db-journal.h"
-#include "tracker-db-backup.h"
 #include "tracker-db-manager.h"
 #include "tracker-db-interface-sqlite.h"
 #include "tracker-db-interface.h"
@@ -53,6 +56,8 @@
 #define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_8
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 
+#define IN_USE_FILENAME               ".meta.isrunning"
+
 typedef enum {
 	TRACKER_DB_LOCATION_DATA_DIR,
 	TRACKER_DB_LOCATION_USER_DATA_DIR,
@@ -141,20 +146,22 @@ static TrackerDBDefinition dbs[] = {
 	  0 },
 };
 
-static gboolean                    db_exec_no_reply    (TrackerDBInterface *iface,
-                                                        const gchar        *query,
-                                                        ...);
+static gboolean            db_exec_no_reply    (TrackerDBInterface *iface,
+                                                const gchar        *query,
+                                                ...);
 static TrackerDBInterface *db_interface_create (TrackerDB           db);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces     (gint num, ...);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces_ro  (gint num, ...);
 
-static gboolean                    initialized;
-static gchar              *sql_dir;
-static gchar              *data_dir;
-static gchar              *user_data_dir;
-static gchar              *sys_tmp_dir;
-static gpointer                    db_type_enum_class_pointer;
-static TrackerDBInterface *resources_iface;
+static gboolean              initialized;
+static gboolean              locations_initialized;
+static gchar                *sql_dir;
+static gchar                *data_dir = NULL;
+static gchar                *user_data_dir = NULL;
+static gchar                *sys_tmp_dir = NULL;
+static gpointer              db_type_enum_class_pointer;
+static TrackerDBInterface   *resources_iface;
+static TrackerDBManagerFlags old_flags = 0;
 
 static const gchar *
 location_to_directory (TrackerDBLocation location)
@@ -292,6 +299,12 @@ function_regexp (TrackerDBInterface *interface,
 	return result;
 }
 
+TrackerDBManagerFlags
+tracker_db_manager_get_flags (void)
+{
+	return old_flags;
+}
+
 static void
 function_group_concat_step (TrackerDBInterface *interface,
                             void               *aggregate_context,
@@ -842,7 +855,7 @@ db_interface_create (TrackerDB db)
 }
 
 static void
-db_manager_remove_all (gboolean rm_backup_and_log, gboolean not_meta)
+db_manager_remove_all (gboolean rm_journal)
 {
 	guint i;
 
@@ -853,33 +866,23 @@ db_manager_remove_all (gboolean rm_backup_and_log, gboolean not_meta)
 	 */
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 
-		if (not_meta && i == TRACKER_DB_METADATA) {
-			continue;
-		}
-
 		g_message ("  Removing database:'%s'",
 		           dbs[i].abs_filename);
 		g_unlink (dbs[i].abs_filename);
 	}
 
-	if (rm_backup_and_log) {
+	if (rm_journal) {
 		GFile *file;
-		gchar *path;
-		const gchar *cpath;
+		gchar *cpath;
 
-		file = tracker_db_backup_file (NULL, TRACKER_DB_BACKUP_META_FILENAME);
-		path = g_file_get_path (file);
-		g_message ("  Removing database:'%s'",
-		           path);
-		g_free (path);
-		g_file_delete (file, NULL, NULL);
-		g_object_unref (file);
-		cpath = tracker_db_journal_filename ();
-		g_message ("  Removing database:'%s'",
+		cpath = g_strdup (tracker_db_journal_get_filename ());
+		tracker_db_journal_shutdown ();
+		g_message ("  Removing journal:'%s'",
 		           cpath);
 		file = g_file_new_for_path (cpath);
 		g_file_delete (file, NULL, NULL);
 		g_object_unref (file);
+		g_free (cpath);
 	}
 }
 
@@ -1019,89 +1022,98 @@ tracker_db_manager_ensure_locale (void)
 	g_free (stored_locale);
 }
 
-static gboolean
-check_meta_backup (gboolean *did_copy)
+static void
+db_recreate_all (void)
 {
-	const gchar *meta_filename;
-	gboolean retval = FALSE;
+	guint i;
 
-	/* This is currently the only test for need_journal. We should add a
-	 * couple tests that test meta.db against consistenty, and if not
-	 * good, copy meta-backup.db over and set need_journal (being less
-	 * conservative about using the backup, and not trusting the meta.db
-	 * as much as we do right now) */
+	/* We call an internal version of this function here
+	 * because at the time 'initialized' = FALSE and that
+	 * will cause errors and do nothing.
+	 */
+	g_message ("Cleaning up database files for reindex");
 
-	meta_filename = dbs[TRACKER_DB_METADATA].abs_filename;
+	db_manager_remove_all (FALSE);
 
-	if (meta_filename) {
-		GFile *file;
+	/* In cases where we re-init this module, make sure
+	 * we have cleaned up the ontology before we load all
+	 * new databases.
+	 */
+	tracker_ontology_shutdown ();
 
-		file = g_file_new_for_path (meta_filename);
+	/* Make sure we initialize all other modules we depend on */
+	tracker_ontology_init ();
 
-		/* (more) Checks for a healthy meta.db should happen here */
+	/* Now create the databases and close them */
+	g_message ("Creating database files, this may take a few moments...");
 
-		if (!g_file_query_exists (file, NULL)) {
-			GFile *backup;
-
-			backup = tracker_db_backup_file (NULL, TRACKER_DB_BACKUP_META_FILENAME);
-
-			if (g_file_query_exists (backup, NULL)) {
-				GError *error = NULL;
-
-				/* Note that we leave meta-backup.db as is, it'll
-				 * be overwritten first-next time tracker-store.c's
-				 * sync_idle_handler will instruct this. */
-
-				g_file_copy (backup, file, G_FILE_COPY_OVERWRITE,
-				             NULL, NULL, NULL, &error);
-
-				if (!error && did_copy) {
-					*did_copy = TRUE;
-				}
-
-				g_clear_error (&error);
-			}
-
-			/* We always play the journal in case meta.db wasn't
-			 * healthy. Also if meta-backup.db didn't exist: that
-			 * just means that tracker-store.c's sync_idle_handler
-			 * didn't yet ran (meanwhile a first log-file is yet
-			 * already being made) */
-
-			retval = TRUE;
-
-			g_object_unref (backup);
-		}
-
-		g_object_unref (file);
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		dbs[i].iface = db_interface_create (i);
 	}
 
-	return retval;
+	/* We don't close the dbs in the same loop as before
+	 * becase some databases need other databases
+	 * attached to be created correctly.
+	 */
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		g_object_unref (dbs[i].iface);
+		dbs[i].iface = NULL;
+	}
+}
+
+void
+tracker_db_manager_init_locations (void)
+{
+	const gchar *dir;
+	guint i;
+	gchar *filename;
+
+	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
+	sys_tmp_dir = g_build_filename (g_get_tmp_dir (), filename, NULL);
+	g_free (filename);
+
+	user_data_dir = g_build_filename (g_get_user_data_dir (),
+	                                  "tracker",
+	                                  "data",
+	                                  NULL);
+
+	data_dir = g_build_filename (g_get_user_cache_dir (),
+	                             "tracker",
+	                             NULL);
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		dir = location_to_directory (dbs[i].location);
+		dbs[i].abs_filename = g_build_filename (dir, dbs[i].file, NULL);
+
+		if (old_flags & TRACKER_DB_MANAGER_LOW_MEMORY_MODE) {
+			dbs[i].cache_size /= 2;
+		}
+	}
+
+	locations_initialized = TRUE;
 }
 
 gboolean
 tracker_db_manager_init (TrackerDBManagerFlags  flags,
                          gboolean              *first_time,
-                         gboolean               shared_cache,
-                         gboolean              *need_journal)
+                         gboolean               shared_cache)
 {
 	GType               etype;
 	TrackerDBVersion    version;
 	gchar              *filename;
 	const gchar        *dir;
 	const gchar        *env_path;
-	gboolean            need_reindex, did_copy = FALSE;
+	gboolean            need_reindex;
 	guint               i;
+	gchar              *in_use_filename;
+	int                 in_use_file;
+	gboolean            loaded = FALSE;
 
 	/* First set defaults for return values */
 	if (first_time) {
 		*first_time = FALSE;
 	}
 
-	if (need_journal) {
-		*need_journal = FALSE;
-	}
-
 	if (initialized) {
 		return TRUE;
 	}
@@ -1123,7 +1135,11 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	/* Set up locations */
 	g_message ("Setting database locations");
 
+	old_flags = flags;
+
 	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
+	if (sys_tmp_dir)
+		g_free (sys_tmp_dir);
 	sys_tmp_dir = g_build_filename (g_get_tmp_dir (), filename, NULL);
 	g_free (filename);
 
@@ -1137,11 +1153,17 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		sql_dir = g_strdup (env_path);
 	}
 
+	if (user_data_dir)
+		g_free (user_data_dir);
+
 	user_data_dir = g_build_filename (g_get_user_data_dir (),
 	                                  "tracker",
 	                                  "data",
 	                                  NULL);
 
+	if (data_dir)
+		g_free (data_dir);
+
 	data_dir = g_build_filename (g_get_user_cache_dir (),
 	                             "tracker",
 	                             NULL);
@@ -1171,6 +1193,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 		/* Fill absolute path for the database */
 		dir = location_to_directory (dbs[i].location);
+		if (dbs[i].abs_filename)
+			g_free (dbs[i].abs_filename);
 		dbs[i].abs_filename = g_build_filename (dir, dbs[i].file, NULL);
 
 		if (flags & TRACKER_DB_MANAGER_LOW_MEMORY_MODE) {
@@ -1195,12 +1219,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		}
 	}
 
-	if (need_journal) {
-		/* That did_copy is used for called db_manager_remove_all, we
-		 * don't want it to also remove our freshly copied meta.db file. */
-
-		*need_journal = check_meta_backup (&did_copy);
-	}
+	locations_initialized = TRUE;
 
 	/* If we are just initializing to remove the databases,
 	 * return here.
@@ -1216,6 +1235,12 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		tracker_db_interface_sqlite_enable_shared_cache ();
 	}
 
+	in_use_filename = g_build_filename (g_get_user_data_dir (),
+	                                    "tracker",
+	                                    "data",
+	                                    IN_USE_FILENAME,
+	                                    NULL);
+
 	/* Should we reindex? If so, just remove all databases files,
 	 * NOT the paths, note, that these paths are also used for
 	 * other things like the nfs lock file.
@@ -1229,53 +1254,103 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 			return FALSE;
 		}
 
-		/* We call an internal version of this function here
-		 * because at the time 'initialized' = FALSE and that
-		 * will cause errors and do nothing.
-		 */
-		g_message ("Cleaning up database files for reindex");
-
-		/* Remove all but the meta.db in case of did_copy, else remove
-		 * all (only in case meta-backup.db was not restored) */
-
-		db_manager_remove_all (FALSE, did_copy);
-
-		/* In cases where we re-init this module, make sure
-		 * we have cleaned up the ontology before we load all
-		 * new databases.
-		 */
-		tracker_ontology_shutdown ();
+		db_recreate_all ();
 
 		/* Make sure we initialize all other modules we depend on */
 		tracker_ontology_init ();
 
-		/* Now create the databases and close them */
-		g_message ("Creating database files, this may take a few moments...");
+		/* Load databases */
+		g_message ("Loading databases files...");
 
+	} else {
+		gboolean must_recreate;
+
+		/* Make sure we initialize all other modules we depend on */
+		tracker_ontology_init ();
+
+		/* Load databases */
+		g_message ("Loading databases files...");
+
+		tracker_db_journal_reader_init (NULL);
+		must_recreate = !tracker_db_journal_reader_verify_last (NULL);
+		tracker_db_journal_reader_shutdown ();
+
+		if (!must_recreate && g_file_test (in_use_filename, G_FILE_TEST_EXISTS)) {
+			gsize size = 0;
+
+			g_message ("Didn't shut down cleanly last time, doing integrity checks");
+
+			for (i = 1; i < G_N_ELEMENTS (dbs) && !must_recreate; i++) {
+				TrackerDBCursor *cursor;
+				TrackerDBStatement *stmt;
+				struct stat st;
+
+				if (g_stat (dbs[i].abs_filename, &st) == 0) {
+					size = st.st_size;
+				}
+
+				/* Size is 1 when using echo > file.db, none of our databases
+				 * are only one byte in size even initually. */
+
+				if (size <= 1) {
+					must_recreate = TRUE;
+					continue;
+				}
+
+				dbs[i].iface = db_interface_create (i);
+				dbs[i].mtime = tracker_file_get_mtime (dbs[i].abs_filename);
+
+				loaded = TRUE;
+
+				stmt = tracker_db_interface_create_statement (dbs[i].iface,
+				                                              "PRAGMA integrity_check(1)");
+
+				cursor = tracker_db_statement_start_cursor (stmt, NULL);
+				g_object_unref (stmt);
+
+				if (cursor) {
+					if (tracker_db_cursor_iter_next (cursor)) {
+						if (g_strcmp0 (tracker_db_cursor_get_string (cursor, 0), "ok") != 0) {
+							must_recreate = TRUE;
+						}
+					}
+					g_object_unref (cursor);
+				}
+			}
+		}
+
+		if (must_recreate) {
+
+			if (first_time) {
+				*first_time = TRUE;
+			}
+
+			for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+				if (dbs[i].iface) {
+					g_object_unref (dbs[i].iface);
+					dbs[i].iface = NULL;
+				}
+			}
+
+			db_recreate_all ();
+			loaded = FALSE;
+		}
+
+	}
+
+	if (!loaded) {
 		for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 			dbs[i].iface = db_interface_create (i);
+			dbs[i].mtime = tracker_file_get_mtime (dbs[i].abs_filename);
 		}
-
-		/* We don't close the dbs in the same loop as before
-		 * becase some databases need other databases
-		 * attached to be created correctly.
-		 */
-		for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-			g_object_unref (dbs[i].iface);
-			dbs[i].iface = NULL;
-		}
-	} else {
-		/* Make sure we initialize all other modules we depend on */
-		tracker_ontology_init ();
 	}
 
-	/* Load databases */
-	g_message ("Loading databases files...");
-
-	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		dbs[i].iface = db_interface_create (i);
-		dbs[i].mtime = tracker_file_get_mtime (dbs[i].abs_filename);
-	}
+	in_use_file = g_open (in_use_filename, 
+	                      O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
+	                      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+	fsync (in_use_file);
+	close (in_use_file);
+	g_free (in_use_filename);
 
 	tracker_db_manager_ensure_locale ();
 
@@ -1293,63 +1368,14 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		                                                        TRACKER_DB_CONTENTS);
 	}
 
-	if (did_copy) {
-		tracker_db_backup_sync_fts ();
-	}
-
 	return TRUE;
 }
 
-void
-tracker_db_manager_disconnect (void)
-{
-	if (resources_iface) {
-		guint i;
-		TrackerDB attachments[2] = { TRACKER_DB_FULLTEXT,
-		                             TRACKER_DB_CONTENTS };
-
-		for (i = 0; i < 2; i++) {
-			TrackerDB db = attachments [i];
-
-			db_exec_no_reply (resources_iface,
-			                  "DETACH '%s'",
-			                  dbs[db].name);
-		}
-
-		tracker_db_interface_disconnect (resources_iface);
-	}
-}
-
-void
-tracker_db_manager_reconnect (void)
-{
-	if (resources_iface) {
-		guint i;
-		TrackerDB attachments[2] = { TRACKER_DB_FULLTEXT,
-		                             TRACKER_DB_CONTENTS };
-
-		tracker_db_interface_reconnect (resources_iface);
-
-		db_set_params (resources_iface,
-		               dbs[TRACKER_DB_METADATA].cache_size,
-		               dbs[TRACKER_DB_METADATA].page_size,
-		               TRUE);
-
-		for (i = 0; i < 2; i++) {
-			TrackerDB db = attachments [i];
-
-			db_exec_no_reply (resources_iface,
-			                  "ATTACH '%s' as '%s'",
-			                  dbs[db].abs_filename,
-			                  dbs[db].name);
-		}
-	}
-}
-
 void
 tracker_db_manager_shutdown (void)
 {
 	guint i;
+	gchar *in_use_filename;
 
 	if (!initialized) {
 		return;
@@ -1368,8 +1394,11 @@ tracker_db_manager_shutdown (void)
 	}
 
 	g_free (data_dir);
+	data_dir = NULL;
 	g_free (user_data_dir);
+	user_data_dir = NULL;
 	g_free (sys_tmp_dir);
+	sys_tmp_dir = NULL;
 	g_free (sql_dir);
 
 	if (resources_iface) {
@@ -1394,14 +1423,108 @@ tracker_db_manager_shutdown (void)
 	tracker_ontology_shutdown ();
 
 	initialized = FALSE;
+	locations_initialized = FALSE;
+
+	in_use_filename = g_build_filename (g_get_user_data_dir (),
+	                                    "tracker",
+	                                    "data",
+	                                    IN_USE_FILENAME,
+	                                    NULL);
+
+	g_unlink (in_use_filename);
+
+	g_free (in_use_filename);
 }
 
 void
-tracker_db_manager_remove_all (gboolean rm_backup_and_log)
+tracker_db_manager_remove_all (gboolean rm_journal)
 {
 	g_return_if_fail (initialized != FALSE);
 
-	db_manager_remove_all (rm_backup_and_log, FALSE);
+	db_manager_remove_all (rm_journal);
+}
+
+
+void
+tracker_db_manager_move_to_temp (void)
+{
+	guint i;
+	gchar *cpath, *new_filename;
+
+	g_return_if_fail (initialized != FALSE);
+
+	g_message ("Moving all database files");
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		g_message ("  Renaming database:'%s' -> '%s'",
+		           dbs[i].abs_filename, new_filename);
+		g_rename (dbs[i].abs_filename, new_filename);
+		g_free (new_filename);
+	}
+
+	cpath = g_strdup (tracker_db_journal_get_filename ());
+	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	g_message ("  Renaming journal:'%s' -> '%s'",
+	           cpath, new_filename);
+	g_rename (cpath, new_filename);
+	g_free (cpath);
+	g_free (new_filename);
+}
+
+
+void
+tracker_db_manager_restore_from_temp (void)
+{
+	guint i;
+	gchar *cpath, *new_filename;
+
+	g_return_if_fail (locations_initialized != FALSE);
+
+	g_message ("Moving all database files");
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		g_message ("  Renaming database:'%s' -> '%s'",
+		           dbs[i].abs_filename, new_filename);
+		g_rename (dbs[i].abs_filename, new_filename);
+		g_free (new_filename);
+	}
+
+	cpath = g_strdup (tracker_db_journal_get_filename ());
+	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	g_message ("  Renaming journal:'%s' -> '%s'",
+	           cpath, new_filename);
+	g_rename (cpath, new_filename);
+	g_free (cpath);
+	g_free (new_filename);
+}
+
+void
+tracker_db_manager_remove_temp (void)
+{
+	guint i;
+	gchar *cpath, *new_filename;
+
+	g_return_if_fail (locations_initialized != FALSE);
+
+	g_message ("Removing all temp database files");
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		new_filename = g_strdup_printf ("%s.tmp", dbs[i].abs_filename);
+		g_message ("  Removing temp database:'%s'",
+		           new_filename);
+		g_unlink (new_filename);
+		g_free (new_filename);
+	}
+
+	cpath = g_strdup (tracker_db_journal_get_filename ());
+	new_filename = g_strdup_printf ("%s.tmp", cpath);
+	g_message ("  Removing temp journal:'%s'",
+	           new_filename);
+	g_unlink (new_filename);
+	g_free (cpath);
+	g_free (new_filename);
 }
 
 void
