@@ -1,5 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* Tracker Extract - extracts embedded metadata from files
+/*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  * Copyright (C) 2008, Nokia
  *
@@ -66,6 +65,10 @@ typedef struct {
 		*fnumber, *flash, *focal_length, *artist,
 		*exposure_time, *iso_speed_ratings, *date, *description,
 		*metering_mode, *creator, *x_dimension, *y_dimension;
+	gchar *city;
+	gchar *state;
+	gchar *address;
+	gchar *country; 
 } TiffNeedsMergeData;
 
 typedef struct {
@@ -381,9 +384,11 @@ extract_tiff (const gchar *uri, TrackerSparqlBuilder *metadata)
 		g_free (exif_data.make);
 	}
 
-	merge_data.title = tracker_coalesce (3, tiff_data.documentname,
-	                                     xmp_data.title, xmp_data.Title,
-	                                     exif_data.document_name);
+	merge_data.title = tracker_coalesce (5, tiff_data.documentname,
+	                                     xmp_data.title, 
+	                                     xmp_data.Title,
+	                                     exif_data.document_name,
+	                                     xmp_data.PDFtitle);
 
 	merge_data.orientation = tracker_coalesce (4, tiff_data.orientation,
 	                                           exif_data.orientation,
@@ -433,6 +438,11 @@ extract_tiff (const gchar *uri, TrackerSparqlBuilder *metadata)
 	merge_data.metering_mode =  tracker_coalesce (2, exif_data.metering_mode,
 	                                              xmp_data.MeteringMode);
 
+	merge_data.city = tracker_coalesce (2, iptc_data.city, xmp_data.City);
+	merge_data.state = tracker_coalesce (2, iptc_data.state, xmp_data.State);
+	merge_data.address = tracker_coalesce (2, iptc_data.sublocation, xmp_data.Address);
+	merge_data.country  = tracker_coalesce (2, iptc_data.countryname, xmp_data.Country);
+
 	merge_data.creator =  tracker_coalesce (3, iptc_data.byline,
 	                                        xmp_data.creator,
 	                                        iptc_data.credit);
@@ -465,11 +475,38 @@ extract_tiff (const gchar *uri, TrackerSparqlBuilder *metadata)
 		g_free (xmp_data.keywords);
 	}
 
+	if (xmp_data.PDFkeywords) {
+		insert_keywords (metadata, uri, xmp_data.PDFkeywords);
+		g_free (xmp_data.PDFkeywords);
+	}
+
 	if (xmp_data.subject) {
 		insert_keywords (metadata, uri, xmp_data.subject);
 		g_free (xmp_data.subject);
 	}
 
+	if (iptc_data.contact) {
+		tracker_sparql_builder_predicate (metadata, "nco:representative");
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nco:Contact");
+		tracker_sparql_builder_predicate (metadata, "nco:fullname");
+		tracker_sparql_builder_object_unvalidated (metadata, iptc_data.contact);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (iptc_data.contact);
+	}
+
+	if (iptc_data.bylinetitle) {
+		tracker_sparql_builder_predicate (metadata, "nco:hasAffiliation");
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nco:Affiliation");
+		tracker_sparql_builder_predicate (metadata, "nco:title");
+		tracker_sparql_builder_object_unvalidated (metadata, iptc_data.bylinetitle);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (iptc_data.bylinetitle);
+	}
+
 	if (xmp_data.publisher) {
 		tracker_sparql_builder_predicate (metadata, "nco:publisher");
 
@@ -531,6 +568,40 @@ extract_tiff (const gchar *uri, TrackerSparqlBuilder *metadata)
 		g_free (xmp_data.license);
 	}
 
+	if (merge_data.city || merge_data.state || merge_data.address || merge_data.country) {
+		tracker_sparql_builder_predicate (metadata, "mlo:location");
+
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
+	
+		if (merge_data.address) {
+			tracker_sparql_builder_predicate (metadata, "mlo:address");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.address);
+			g_free (merge_data.address);
+		}
+	
+		if (merge_data.state) {
+			tracker_sparql_builder_predicate (metadata, "mlo:state");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.state);
+			g_free (merge_data.state);
+		}
+	
+		if (merge_data.city) {
+			tracker_sparql_builder_predicate (metadata, "mlo:city");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.city);
+			g_free (merge_data.city);
+		}
+	
+		if (merge_data.country) {
+			tracker_sparql_builder_predicate (metadata, "mlo:country");
+			tracker_sparql_builder_object_unvalidated (metadata, merge_data.country);
+			g_free (merge_data.country);
+		}
+		
+		tracker_sparql_builder_object_blank_close (metadata);
+	}
+
 	if (iptc_data.keywords) {
 		insert_keywords (metadata, uri, iptc_data.keywords);
 		g_free (iptc_data.keywords);
